//
//  ViewController.m
//  CalculatorTest
//
//  Created by Daniel Grassi on 1/30/12.
//  Copyright (c) 2012 Zaph, LLC. All rights reserved.
//

#import "ViewController.h"

@interface ViewController ()
@property (strong, nonatomic) IBOutlet UILabel *registerDisplay;
@property (strong, nonatomic) IBOutlet UIImageView *displaySurroundImageView;
@property (strong, nonatomic) IBOutlet UIView *buttonContainerView;

@property (strong, nonatomic) NSDecimalNumber *secondOperand;
@property (strong, nonatomic) NSDecimalNumber *firstOperand;
@property (strong, nonatomic) NSValue *operationSelector;
@property (assign, nonatomic, getter=isEntryInprogress) BOOL entryInprogress;
@property (strong, nonatomic) NSMutableString *entryString;
@property (assign, nonatomic) BOOL isNumber;

@end

@implementation ViewController

#pragma mark View methods
- (void)viewDidLoad {
    [super viewDidLoad];
    
    [self addBackgroundToButtons];
    [self addBackgroundToDisplay];
    self.entryInprogress = NO;
    self.entryString = [NSMutableString string];
    [self clearDisplay];
}

- (void)addBackgroundToButtons {
    UIImage *buttomBackgroungImage = [UIImage imageNamed:@"ButtonBackground"];
    UIEdgeInsets capInsets = UIEdgeInsetsMake(8, 8, 8, 8);
    UIImage *resizableImage = [buttomBackgroungImage resizableImageWithCapInsets:capInsets];
    
    for (UIButton *button in self.buttonContainerView.subviews) {
        [button setBackgroundImage:resizableImage forState:UIControlStateNormal];
    }
}

- (void)addBackgroundToDisplay {
    UIImage *buttomBackgroungImage = [UIImage imageNamed:@"DisplayBackground"];
    UIEdgeInsets capInsets = UIEdgeInsetsMake(8, 8, 8, 8);
    UIImage *resizableImage = [buttomBackgroungImage resizableImageWithCapInsets:capInsets];
    
    self.displaySurroundImageView.image = resizableImage;
}

- (void)viewDidUnload {
    self.registerDisplay = nil;
    self.displaySurroundImageView = nil;
    [self setButtonContainerView:nil];
    [super viewDidUnload];
}

#pragma mark Calulator methods
- (void)clearDisplay {
    self.secondOperand = NSDecimalNumber.zero;
    self.firstOperand  = NSDecimalNumber.zero;
    self.isNumber = YES;
    self.entryInprogress = NO;
    [self.entryString setString:@""];
    [self updateDisplayRegisters];
}

- (void)updateDisplayRegisters {
    if (self.isNumber == YES) {
        self.registerDisplay.text = [self formatNumber:self.firstOperand];
    }
    else {
        self.registerDisplay.text = @"Not a number";
    }
}

#pragma mark Keypad button handling
- (IBAction)numericButton:(UIButton *)sender {
    if (self.isEntryInprogress == NO) {
        self.secondOperand = self.firstOperand;
        [self.entryString setString:@""];
        self.entryInprogress = YES;
        self.isNumber = YES;
    }

    [self.entryString appendString:sender.currentTitle];
    self.firstOperand = [NSDecimalNumber decimalNumberWithString:self.entryString];
    [self updateDisplayRegisters];
}

- (IBAction)decimalPointButton:(UIButton *)sender {
    [self numericButton:sender];
}

- (IBAction)addButton:(id)sender {
    self.operationSelector = [NSValue valueWithPointer:@selector(addOperation)];
    self.entryInprogress = NO;
    self.secondOperand = [self.secondOperand decimalNumberByAdding:self.firstOperand];
}

- (IBAction)subtractButton:(id)sender {
    self.operationSelector = [NSValue valueWithPointer:@selector(subtractOperation)];
    self.entryInprogress = NO;
    self.secondOperand = [self.secondOperand decimalNumberBySubtracting:self.firstOperand];
}

- (IBAction)multipleButton:(id)sender {
    self.operationSelector = [NSValue valueWithPointer:@selector(multipleOperation)];
    self.entryInprogress = NO;
    self.secondOperand = [self.secondOperand decimalNumberByMultiplyingBy:self.firstOperand];
}

- (IBAction)divideButton:(id)sender {
    self.operationSelector = [NSValue valueWithPointer:@selector(divideOperation)];
    self.entryInprogress = NO;
    self.secondOperand = [self.secondOperand decimalNumberByDividingBy:self.firstOperand];
}

- (IBAction)equalButton:(id)sender {
	SEL selector = [self.operationSelector pointerValue];
    
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Warc-performSelector-leaks"
    if ([self respondsToSelector:selector]) {
        [self performSelector:selector];
    }
#pragma clang diagnostic pop
    [self updateDisplayRegisters];
    
    self.entryInprogress = NO;
    [self.entryString setString:@""];
}

- (IBAction)clearButton:(id)sender {
    [self clearDisplay];
}

- (IBAction)deleteButton:(id)sender {
    
    if (self.isEntryInprogress == YES &&
        self.entryString.length > 0) {
        [self.entryString deleteCharactersInRange:NSMakeRange(self.entryString.length-1, 1)];
        [self updateDisplayRegisters];
    }
}

#pragma mark Numeric utilities
- (NSString *)formatNumber:(NSDecimalNumber *)number {
    NSNumberFormatter *numberFormatter = [NSNumberFormatter new];
    [numberFormatter setMaximumFractionDigits:6];
    [numberFormatter setMinimumIntegerDigits:1];
    
    return [numberFormatter stringFromNumber:number];
}

- (BOOL)isNumber:(NSDecimalNumber *)number {
    NSComparisonResult isZero = [number compare:NSDecimalNumber.zero];
    return (isZero != NSOrderedSame) ? YES : NO;
}

#pragma mark Numeric operations
- (NSDecimalNumber *)addOperation {
    return self.firstOperand = [self.firstOperand decimalNumberByAdding:self.secondOperand];
}

- (NSDecimalNumber *)subtractOperation {
    return self.firstOperand = [self.secondOperand decimalNumberBySubtracting:self.firstOperand];
}

- (NSDecimalNumber *)multipleOperation {
    return self.firstOperand = [self.firstOperand decimalNumberByMultiplyingBy:self.secondOperand];
}

- (NSDecimalNumber *)divideOperation {
    if ([self isNumber:self.firstOperand]) {
        return self.firstOperand = [self.secondOperand decimalNumberByDividingBy:self.firstOperand];
    }
    else {
        self.isNumber = NO;
        return self.firstOperand = NSDecimalNumber.zero;
    }
}

@end
