//
//  ViewController.m
//  Calculator
//
//  Created by Daniel Grassi on 1/30/12.
//  Copyright (c) 2012 Zaph, LLC. All rights reserved.
//

#import "ViewController.h"

@interface ViewController ()
@property (strong, nonatomic) IBOutlet UILabel *tRegisterDisplay;
@property (strong, nonatomic) IBOutlet UILabel *zRegisterDisplay;
@property (strong, nonatomic) IBOutlet UILabel *yRegisterDisplay;
@property (strong, nonatomic) IBOutlet UILabel *xRegisterDisplay;
@property (strong, nonatomic) IBOutlet UIImageView *displaySurroundImageView;
@property (strong, nonatomic) IBOutlet UIView *buttonContainerView;

@property (strong, nonatomic) NSDecimalNumber *t;
@property (strong, nonatomic) NSDecimalNumber *z;
@property (strong, nonatomic) NSDecimalNumber *y;
@property (strong, nonatomic) NSDecimalNumber *x;
@property (assign, nonatomic, getter=isEntryInprogress) BOOL entryInprogress;
@property (assign, nonatomic, getter=isResultDisplayed) BOOL resultDisplayed;
@property (strong, nonatomic) NSMutableString *entryString;
@property (assign, nonatomic, getter=isNotNumber) BOOL notNumber;

@property (strong, nonatomic) NSMutableArray *stack;

@end

@implementation ViewController

- (void)viewDidLoad {
    [super viewDidLoad];

    self.stack = [NSMutableArray array];
    [self addBackgroundToButtons];
    [self addBackgroundToDisplay];
    self.entryInprogress = NO;
    self.entryString = [NSMutableString string];
    [self clearStack];
}

- (void)addBackgroundToButtons {
    UIImage *buttomBackgroungImage = [UIImage imageNamed:@"ButtonBackground"];
    UIEdgeInsets capInsets = UIEdgeInsetsMake(8, 8, 8, 8);
    UIImage *resizableImage = [buttomBackgroungImage resizableImageWithCapInsets:capInsets];
    
    for (UIButton *button in self.buttonContainerView.subviews) {
        [button setBackgroundImage:resizableImage forState:UIControlStateNormal];
    }
}

- (void)addBackgroundToDisplay {
    UIImage *buttomBackgroungImage = [UIImage imageNamed:@"DisplayBackground"];
    UIEdgeInsets capInsets = UIEdgeInsetsMake(8, 8, 8, 8);
    UIImage *resizableImage = [buttomBackgroungImage resizableImageWithCapInsets:capInsets];
    
    self.displaySurroundImageView.image = resizableImage;
}

- (void)viewDidUnload
{
    self.tRegisterDisplay=nil;
    self.zRegisterDisplay=nil;
    self.yRegisterDisplay=nil;
    self.xRegisterDisplay=nil;
    [self setDisplaySurroundImageView:nil];
    [self setButtonContainerView:nil];
    [super viewDidUnload];
}

- (void)updateDisplayRegisters {
    self.tRegisterDisplay.text = [self formatNumber:self.t];
    self.zRegisterDisplay.text = [self formatNumber:self.z];
    self.yRegisterDisplay.text = [self formatNumber:self.y];

    if (self.isNotNumber == NO) {
        self.xRegisterDisplay.text = [self formatNumber:self.x];
    }
    else {
        self.xRegisterDisplay.text = @"Not a number";
    }
}

- (IBAction)numericButton:(UIButton *)sender {
    if (self.isEntryInprogress == NO) {
        if (self.isResultDisplayed == YES) {
            [self pushStack:NSDecimalNumber.zero];
        }
        self.notNumber = NO;
        [self.entryString setString:@""];
        self.entryInprogress = YES;
    }

    [self.entryString appendString:sender.currentTitle];
    self.x = [NSDecimalNumber decimalNumberWithString:self.entryString];
    [self updateDisplayRegisters];
}
- (IBAction)decimalPointButton:(UIButton *)sender {
    [self numericButton:sender];
}
- (IBAction)addButton:(id)sender {
    self.entryInprogress = NO;
    self.y = [self.y decimalNumberByAdding:self.x];
    [self dropStack];
}
- (IBAction)subtractButton:(id)sender {
    self.entryInprogress = NO;
    self.y = [self.y decimalNumberBySubtracting:self.x];
    [self dropStack];
}
- (IBAction)multipleButton:(id)sender {
    self.entryInprogress = NO;
    self.y = [self.y decimalNumberByMultiplyingBy:self.x ];
    [self dropStack];
}
- (IBAction)divideButton:(id)sender {
    self.entryInprogress = NO;
    if ([self isNumber:self.x]) {
        self.y = [self.y decimalNumberByDividingBy:self.x];
    }
    else {
        self.notNumber = YES;
    }
    [self dropStack];
}

- (IBAction)enterButton:(id)sender {
    [self pushStack:self.x];
    self.entryInprogress = NO;
    [self.entryString setString:@""];
}

- (IBAction)deleteLastDigitButton:(id)sender {
    if (self.isEntryInprogress == YES &&
        self.entryString.length > 0) {
        [self.entryString deleteCharactersInRange:NSMakeRange(self.entryString.length-1, 1)];
        self.x = [NSDecimalNumber decimalNumberWithString:self.entryString];
        [self updateDisplayRegisters];
    }
}

- (IBAction)dropStack:(id)sender {
    [self dropStack];
}

- (IBAction)swapXYRegistersButton:(id)sender {
    if (self.stack.count < 2) {
        return;
    }

    [self.stack exchangeObjectAtIndex:0 withObjectAtIndex:1];
    [self updateDisplayRegisters];
}

- (NSString *)formatNumber:(NSDecimalNumber *)number {
    NSNumberFormatter *numberFormatter = [NSNumberFormatter new];
    [numberFormatter setMaximumFractionDigits:6];
    [numberFormatter setMinimumIntegerDigits:1];
    
    return [numberFormatter stringFromNumber:number];
}

- (BOOL)isNumber:(NSDecimalNumber *)number {
    NSComparisonResult isZero = [number compare:NSDecimalNumber.zero];
    return (isZero != NSOrderedSame) ? YES : NO;
}

- (void)clearStack {
    [self.stack removeAllObjects];
    self.notNumber = NO;
    self.entryInprogress = NO;
    [self.entryString setString:@""];
    [self updateDisplayRegisters];
    self.resultDisplayed = NO;
}

- (void)pushStack:(NSDecimalNumber *)value {
    [self.stack insertObject:value atIndex:0];
    [self updateDisplayRegisters];
    self.resultDisplayed = NO;
}

- (void)dropStack {
    if (self.stack.count > 0) {
        [self.stack removeObjectAtIndex:0];
        [self updateDisplayRegisters];
    }
    self.resultDisplayed = YES;
    self.entryInprogress = NO;
}

- (void)setX:(NSDecimalNumber *)x {
    [self replaceValue:x AtIndex:0];
}
- (NSDecimalNumber *)x {
    return [self valueAtIndex:0];
}

- (void)setY:(NSDecimalNumber *)y {
    [self replaceValue:y AtIndex:1];
}
- (NSDecimalNumber *)y {
    return [self valueAtIndex:1];
}

- (void)setZ:(NSDecimalNumber *)z {
    [self replaceValue:z AtIndex:2];
}
- (NSDecimalNumber *)z {
    return [self valueAtIndex:2];
}

- (void)setT:(NSDecimalNumber *)t {
    [self replaceValue:t AtIndex:3];
}
- (NSDecimalNumber *)t {
    return [self valueAtIndex:3];
}

- (void)replaceValue:(NSDecimalNumber *)value AtIndex:(NSUInteger)index {
    while (index >= self.stack.count) {
        [self.stack addObject:NSDecimalNumber.zero];
    }
    [self.stack replaceObjectAtIndex:index withObject:value];
}

- (NSDecimalNumber *)valueAtIndex:(NSUInteger)index {
    if (index < self.stack.count) {
        return [self.stack objectAtIndex:index];
    }
    return NSDecimalNumber.zero;
}

@end
